<?php

namespace Drupal\Tests\drowl_media\Functional;

use Drupal\Core\Config\Entity\ConfigEntityInterface;
use Drupal\field\Entity\FieldConfig;
use Drupal\field\Entity\FieldStorageConfig;
use Drupal\media\Entity\Media;
use Drupal\Tests\BrowserTestBase;
use Drupal\Tests\media\Traits\MediaTypeCreationTrait;

/**
 * Tests drowl_media.
 *
 * @group drowl_media
 */
class DrowlMediaFunctionalTests extends BrowserTestBase {
  use MediaTypeCreationTrait;
  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'drowl_media',
    'test_page_test',
    'blazy',
    'crop',
    'dropzonejs_eb_widget',
    'embed',
    'entity_browser',
    'entity_browser_entity_form',
    'entity_embed',
    'media_entity_file_replace',
    'fences',
    'field',
    'field_group',
    'file',
    'focal_point',
    'image',
    'inline_entity_form',
    'language',
    'link',
    'metatag',
    'node',
    'options',
    'path',
    'pathauto',
    'rabbit_hole',
    'responsive_image',
    'smart_trim',
    'taxonomy',
    'text',
    'user',
    'views',
    'views_linkarea',
    'viewsreference',
  ];

  /**
   * A user with authenticated permissions.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $user;

  /**
   * A user with admin permissions.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $adminUser;

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->config('system.site')->set('page.front', '/test-page')->save();
    $this->user = $this->drupalCreateUser([]);
    $this->adminUser = $this->drupalCreateUser([]);
    $this->adminUser->addRole($this->createAdminRole('admin', 'admin'));
    $this->adminUser->save();
    $this->drupalLogin($this->adminUser);
  }

  /**
   * Tests if the module installation, won't break the site.
   */
  public function testInstallation() {
    $session = $this->assertSession();
    $this->drupalGet('<front>');
    $session->statusCodeEquals(200);
  }

  /**
   * Tests if the drupal media library page is accessible.
   */
  public function testMediaLibraryExists() {
    $session = $this->assertSession();
    $this->drupalGet('/admin/content/media');
    $session->statusCodeEquals(200);
  }


/**
   * Tests if the media library exists in the header of the media overview.
   */
  public function testDrowlMediaLibraryAdminLibraryExists() {
    $session = $this->assertSession();
    $this->drupalGet('/admin/content/media');
    $session->statusCodeEquals(200);
    $session->elementExists('css', 'head > link[href*=drowl_media]');
  }
}
